// Generated by CoffeeScript 1.6.3
var Base, Client, dnode, helper, transports, _,
  __hasProp = {}.hasOwnProperty,
  __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

_ = require('../vendor/lodash');

dnode = require('dnode');

Base = require('./base');

helper = require('./helper');

transports = require('./transports');

Client = (function(_super) {
  __extends(Client, _super);

  Client.prototype.name = 'Client';

  Client.prototype.defaults = {
    debug: true,
    maxRetries: 5,
    timeout: 5000,
    interval: 1000,
    port: 7337
  };

  function Client() {
    Client.__super__.constructor.apply(this, arguments);
    this.count = {
      ping: 0,
      pong: 0,
      attempt: 0
    };
    this.connecting = false;
    this.status = 'down';
    this.reconnect = _.throttle(this.reconnect, this.opts.interval, {
      leading: true
    });
    this.ping = _.throttle(this.ping, this.opts.interval);
    this.bindTo = this.bind;
  }

  Client.prototype.bind = function() {
    this.count.attempt = 0;
    return this.ci = transports.bind(this, arguments);
  };

  Client.prototype.unbind = function() {
    this.count.attempt = 0;
    return this.reset();
  };

  Client.prototype.createConnection = function(fn) {
    if (typeof fn !== 'function') {
      this.err("must be a function");
    }
    if (!(fn.length === 1 || fn.length === 2)) {
      this.err("must have arity 1 or 2");
    }
    this.getConnectionFn = fn;
    return this.reconnect();
  };

  Client.prototype.server = function(callback) {
    if (!this.getConnectionFn) {
      return this.err("no create connection method defined");
    }
    if (this.status === 'up') {
      return callback(this.remote);
    } else if (this.status === 'down' && !this.connecting) {
      this.count.attempt = 0;
      this.reconnect();
    }
    return this.once('remote', callback);
  };

  Client.prototype.unget = function(callback) {
    return this.removeListener('remote', callback);
  };

  Client.prototype.reconnect = function() {
    var _this = this;
    if (this.status === 'up' || this.connecting || this.count.attempt >= this.opts.maxRetries) {
      return;
    }
    this.count.attempt++;
    this.connecting = true;
    this.reset();
    this.d = dnode(this.exposed);
    this.d.once('remote', this.onRemote);
    this.d.once('end', this.onEnd);
    this.d.once('error', this.onError);
    this.d.once('fail', this.onStreamError);
    this.timeout(function() {
      _this.reset();
      return _this.reconnect();
    });
    this.emit('connecting');
    switch (this.getConnectionFn.length) {
      case 1:
        return this.getConnectionFn(function(stream) {
          if (!helper.isReadable(stream)) {
            _this.err("Invalid duplex stream (not readable)");
          }
          if (!helper.isWritable(stream)) {
            _this.err("Invalid duplex stream (not writable)");
          }
          stream.on('error', _this.onStreamError);
          return stream.pipe(_this.d).pipe(stream);
        });
      case 2:
        return this.getConnectionFn(function(read) {
          if (!helper.isReadable(read)) {
            _this.err("Invalid read stream");
          }
          read.on('error', _this.onStreamError);
          return read.pipe(_this.d);
        }, function(write) {
          if (!helper.isWritable(write)) {
            _this.err("Invalid write stream");
          }
          write.on('error', _this.onStreamError);
          return _this.d.pipe(write);
        });
    }
  };

  Client.prototype.onStreamError = function(err) {
    this.setStatus('down');
    return this.reconnect();
  };

  Client.prototype.onError = function(err) {
    this.log("error: " + err);
    return this.emit("error", err);
  };

  Client.prototype.onRemote = function(remote) {
    var _ref;
    this.timeout(false);
    if (!((_ref = remote._pnode) != null ? _ref.ping : void 0)) {
      return this.err("Invalid pnode host");
    }
    this.remote = remote;
    this.emit('remote', this.remote, this);
    this.setStatus('up');
    return this.ping();
  };

  Client.prototype.ping = function() {
    var _this = this;
    if (this.status === 'down') {
      return;
    }
    this.count.ping++;
    this.timeout(true);
    return this.remote._pnode.ping(function(ok) {
      if (ok === true) {
        _this.count.pong++;
      }
      _this.timeout(false);
      return _this.ping();
    });
  };

  Client.prototype.timeout = function(cb) {
    var _this = this;
    clearTimeout(this.timeout.t);
    if (cb === false) {
      return;
    }
    return this.timeout.t = setTimeout(function() {
      _this.setStatus('down');
      if (typeof cb === 'function') {
        return cb();
      }
    }, this.opts.timeout);
  };

  Client.prototype.onEnd = function() {
    this.log("server closed connection");
    this.setStatus('down');
    return this.reconnect();
  };

  Client.prototype.setStatus = function(s) {
    this.connecting = false;
    if (!((s === 'up' || s === 'down') && s !== this.status)) {
      return;
    }
    this.log(s);
    this.status = s;
    return this.emit(s);
  };

  Client.prototype.reset = function() {
    this.setStatus('down');
    if (this.d) {
      return this.d.removeAllListeners().end();
    }
  };

  return Client;

})(Base);

module.exports = function(opts) {
  return new Client(opts);
};
